package com.sap
{
	import mx.rpc.http.mxml.HTTPService;
	import flash.events.IEventDispatcher;
	import flash.events.EventDispatcher;
	import flash.events.Event;
	import mx.rpc.events.ResultEvent;
	import mx.rpc.events.FaultEvent;
	import com.sap.SAPWFFaultEvent;
	import com.sap.SAPWFResultEvent;	
	[Event(name="result", type="SAPWFResultEvent")]
	[Event(name="fault", type="SAPWFFaultEvent")]	
	public class SAPWFHelper implements IEventDispatcher	
	{
		private var _requestParamsInput:String;
		private var _requestParamsStructure:XML;
		private var _requestParameterTable:XML;
		private var _tabHashMap:Object;
		private var _rowHashMap:Object;
		private var _sapwfService:HTTPService;
		private var _dispatcher:EventDispatcher;
		public function SAPWFHelper():void{
			_dispatcher = new EventDispatcher(this);
			_sapwfService = new HTTPService();
			_requestParamsInput = new String();
			_tabHashMap = new Object();
			_rowHashMap = new Object();
			_requestParamsStructure = new XML("<STRUCT/>");
			_requestParameterTable = new XML("<TABLE/>");
		}
	    public function addEventListener(type:String, listener:Function, useCapture:Boolean = false, priority:int = 0.0, useWeakReference:Boolean = false):void{
	        _dispatcher.addEventListener(type, listener, useCapture, priority);
	    }
	    public function dispatchEvent(evt:Event):Boolean{
	        return _dispatcher.dispatchEvent(evt);
	    }
	    
	    public function hasEventListener(type:String):Boolean{
	        return _dispatcher.hasEventListener(type);
	    }
	    
	    public function removeEventListener(type:String, listener:Function, useCapture:Boolean = false):void{
	        _dispatcher.removeEventListener(type, listener, useCapture);
	    }
	                   
	    public function willTrigger(type:String):Boolean {
	        return _dispatcher.willTrigger(type);
	    }	    		
		public function sendRequestToSAPWF(BAPIName:String, serviceProvider:String=null):void{
			var serviceURL:String = "http://localhost:7531/SapMiddleTier/REST/ewcs/service/rfct";
			if (serviceProvider != null){
				serviceURL += "?serviceProvider=" + serviceProvider;
			}
			_sapwfService.url = serviceURL
			_sapwfService.method = "POST";
			_sapwfService.contentType = "application/xml";	
			_sapwfService.request = createRequest(BAPIName);
			_sapwfService.addEventListener(ResultEvent.RESULT, httpResult);
			_sapwfService.addEventListener(FaultEvent.FAULT, httpFault);
			_sapwfService.send();

		}
		private function httpResult(event:ResultEvent):void{
			dispatchEvent(new SAPWFResultEvent(SAPWFResultEvent.RESULT, event.result));
		}
		private function httpFault(event:FaultEvent):void{
			dispatchEvent(new SAPWFFaultEvent(SAPWFFaultEvent.FAULT, event.fault));
		}		
		private function createRequest(BAPIName:String):XML{
		   var i:int;
		   var xmlHeader:String = "<root> " + 
			      			"<RfcRequest name='" + BAPIName + "'> " + 
			      			"<![CDATA[ ";
		   var xmlFooter:String = "]]> " + 
			      			"</RfcRequest>" + 
			      			"</root>"; 
			if (_requestParamsInput.length != 0){
				xmlHeader += _requestParamsInput;
			}
			if (_requestParamsStructure.children().length() != 0){
				if(_requestParamsInput.length != 0){
				for(i = 0; i < _requestParamsStructure.children().length(); i++){
					xmlHeader += "," + _requestParamsStructure.children()[i].toXMLString();
				}				
				}else{
				xmlHeader += _requestParamsStructure.children()[0].toXMLString()
				for(i = 1; i < _requestParamsStructure.children().length(); i++){
					xmlHeader += "," + _requestParamsStructure.children()[i].toXMLString();
				}					
				}
			}
			if (_requestParameterTable.children().length() != 0){
				if ((_requestParamsInput.length != 0) || (_requestParamsInput.length != 0)){
				for(i = 0; i < _requestParameterTable.children().length(); i++){
					xmlHeader += "," + _requestParameterTable.children()[i].toXMLString();
				}
				}else{
					xmlHeader += _requestParameterTable.children()[0].toXMLString()
				for(i = 1; i < _requestParameterTable.children().length(); i++){
					xmlHeader += "," + _requestParameterTable.children()[i].toXMLString();
				}
				}
			}
			var xmlString:String = xmlHeader + xmlFooter;			
			      		   XML.ignoreWhitespace;  				
      			return new XML(xmlString);	
		}
		public function addParamInput(paramName:String, paramValue:String):void{
			if (_requestParamsInput.length == 0){
			_requestParamsInput += "<" + paramName + ">" + paramValue + "</" + paramName + ">";
			}else{
			_requestParamsInput += ",<" + paramName + ">" + paramValue + "</" + paramName + ">";				
			}
			
		}
		public function addParamStructure(structName:String, fieldName:String, value:String):void{
			var tempString:String;
			var childList:XMLList = _requestParamsStructure.child(structName);
			if(childList.length() == 0){
				tempString = "<" + structName + ">" + "<" + fieldName + ">" + value + "</" + fieldName + ">" + "</" + structName + ">";
				XML.ignoreWhitespace;
				_requestParamsStructure.appendChild(XML(tempString));
			}else{
				tempString = "<" + fieldName + ">" + value + "</" + fieldName + ">";
				_requestParamsStructure.children()[0].appendChild(new XML(tempString));
			}
			
		}
		public function addTableToParams(tabName:String):void{
			var tabToAdd:TableWF = _tabHashMap[tabName];
			_requestParameterTable.appendChild(tabToAdd.tableXML);
			
		}
		public function createParamTable(tabName:String):void{
			var newTable:TableWF = new TableWF(tabName);
			_tabHashMap[tabName] = newTable;
		}
		public function addRowToTable(tabName:String, rowName:String):void{
			var myRow:TableWFRow = _rowHashMap[rowName];
			var myTab:TableWF = _tabHashMap[tabName];
			if ((myRow != null)&&(myTab != null)){
				myTab.addRow(myRow.rowXML);
			}
		}
		public function createRow(rowName:String):void{
			var newRow:TableWFRow = new TableWFRow();
			_rowHashMap[rowName] = newRow;
		}
		public function addRowField(rowName:String, fieldName:String, value:String):void{
			var row:TableWFRow =  _rowHashMap[rowName];
			if(row != null){
				row.addField(fieldName, value);
			}
		}				
	}
}

class TableWF
{
	private var _tableXML:XML;
	public function get tableXML():XML{
		return _tableXML;
	}
	public function TableWF(tableName:String){
		_tableXML = new XML("<" + tableName + "/>");
	}
	public function addRow(row:XML):void{
		_tableXML.appendChild(row);
	}	
}

class TableWFRow
{
	private var _rowXML:XML;
	public function get rowXML():XML{
		return _rowXML;
	}
	public function TableWFRow():void{
		var tempString:String = "<ROW/>";
		_rowXML = new XML(tempString);
	}
	public function addField(fieldName:String, fieldValue:String):void{
		var tempString:String = "<" + fieldName + ">" + fieldValue + "</" + fieldName + ">";
		_rowXML.appendChild(new XML(tempString));
	}
}
   